#include <windows.h>   
#include <math.h>   

const float MISSING = (float) 3.4e38;


_declspec(dllexport) void DMH (float *High, float *Low, long Length, long numrows, float *output )
{
	//DMH - Directional Movement using Hann Windowing
	//(C) 2021 John F. Ehlers

	double SF=0.0, PlusDM=0.0, MinusDM=0.0;
	double UpperMove=0.0, LowerMove=0.0, *EMA;
	double DMSum=0.0, coef=0.0, DMH=0.0;
	long CurrentBar, count=0;

	double radiansconvertmult;
	radiansconvertmult = 3.1415926535 / 180.0;

	EMA = malloc ( sizeof(double) * numrows );
	EMA[0] = 0.0;

	for ( CurrentBar=1; CurrentBar<numrows; CurrentBar++ )  // start at 1 instead of 0 for upper/lower move calcs with lag of 1
	{	SF = 1.0 / Length;
		UpperMove = High[CurrentBar] - High[CurrentBar-1];
		LowerMove = Low[CurrentBar-1] - Low[CurrentBar];
		PlusDM = 0;
		MinusDM = 0;

		if ( UpperMove > LowerMove && UpperMove > 0 )
			PlusDM = UpperMove;
		else if ( LowerMove > UpperMove && LowerMove > 0 )
			MinusDM = LowerMove;

		EMA[CurrentBar] = SF*(PlusDM - MinusDM) + (1 - SF)* EMA[CurrentBar - 1];

		if ( CurrentBar >= Length )
		{
			//Smooth Directional Movements with Hann Windowed FIR filter
			DMSum = 0;
			coef = 0;
			for ( count=1; count<=Length; count++ )
			{	DMSum = DMSum + (1 - cos(radiansconvertmult*360*count / (Length + 1)))*EMA[CurrentBar - (count - 1)];
				coef = coef + (1 - cos(radiansconvertmult*360*count / (Length + 1)));
			}

			if ( coef != 0 ) DMH = DMSum / coef;
			output[CurrentBar] = (float) DMH;
		}
	}

	free(EMA);
}
